VERSION 5.00
Begin {C62A69F0-16DC-11CE-9E98-00AA00574A4F} frmWebSearchLauncher 
   Caption         =   "Web Search Launcher"
   ClientHeight    =   4770
   ClientLeft      =   120
   ClientTop       =   465
   ClientWidth     =   3930
   OleObjectBlob   =   "frmWebSearchLauncher.frx":0000
   StartUpPosition =   1  'CenterOwner
End
Attribute VB_Name = "frmWebSearchLauncher"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
Option Explicit

' 2020-08-31
' Macro concept and some code is taken from GoogleFetch Version 08.07.20 by Paul Beverley
' File verification code taken from FileThere macro -- https://word.tips.net/T000819_Determining_if_a_File_Exists.html
' Written by Michael Schuler (https://bergerplusschuler.com)

' ******************
' Private Variables
' ******************

Private Const sDIALOGUE_TITLE As String = "Web Search Launcher"
Private Const sDATA_FILE As String = "WebSearchLauncherData.docx"
Private sSelection As String

' ******************
' Functions
' ******************

Private Function RemoveBadCharacters(sString As String) As String
    'Removes problematic characters from a search string
    
    Dim iLoop As Integer

    sString = Replace(sString, Chr(30), "-") ' replace nonbreaking hyphen with hyphen - is this needed?
    For iLoop = 1 To 31
        sString = Replace(sString, Chr(iLoop), " ") 'convert special characters to spaces
    Next iLoop
    Do While (InStr(sString, "  "))
        sString = Replace(sString, "  ", " ") ' remove extra spaces
    Loop
    RemoveBadCharacters = sString

End Function


Private Sub TextSearchString_Change()

End Sub

' ******************
' Subroutines
' ******************

Private Sub UserForm_Initialize()

    Dim sWebsiteName() As String
    Dim sWebsiteURL() As String
    Dim dSourceDoc As Document
    Dim aData() As String
    Dim iRows As Integer
    Dim rCellRange As Range
    Dim lRow As Long
    Dim lColumn As Long
    Dim sSourcePath As String
    
    On Error GoTo Err_Msg
    
    'Verify source data file exists; if not, create a basic array and disable the website list button
    sSourcePath = ActiveDocument.AttachedTemplate.Path & Application.PathSeparator & sDATA_FILE
    If Not (Dir(sSourcePath) > "") Then
        MsgBox "The website source file " & Chr(34) & sDATA_FILE & Chr(34) & " was not found. Default websites will be provided.", vbCritical, sDIALOGUE_TITLE
        ReDim aData(1, 1)
        aData(0, 0) = "Merriam-Webster Online"
        aData(0, 1) = "https://www.merriam-webster.com/dictionary/"
        aData(1, 0) = "Google"
        aData(1, 1) = "http://www.google.com/search?q="
        cmdOpenWebsiteList.Enabled = False
        Else
            Set dSourceDoc = Documents.Open(FileName:=sSourcePath, Visible:=False)
            iRows = dSourceDoc.tables(1).Rows.Count - 1
            ReDim aData(iRows - 1, 1)
            For lColumn = 0 To 1
                For lRow = 0 To iRows - 1
                    Set rCellRange = dSourceDoc.tables(1).cell(lRow + 2, lColumn + 1).Range
                    rCellRange.End = rCellRange.End - 1
                    aData(lRow, lColumn) = rCellRange.Text
                Next lRow
            Next lColumn
            dSourceDoc.Close savechanges:=wdDoNotSaveChanges
    End If
    
    'Use the .List property to populate the listbox with the array data
    lstWebsites.List = aData
    lstWebsites.ListIndex = 0
    
    'Prepare the search string
    If Len(Selection) = 1 Then Selection.Expand wdWord
    sSelection = Trim(Selection)
    TextSearchString.Text = RemoveBadCharacters(sSelection)
    lstWebsites.SetFocus
       
Exit Sub
    
Err_Msg:
    MsgBox "The macro has encountered an error." & vbCrLf & Err.Number & ": " & Err.description, vbCritical, sDIALOGUE_TITLE
  
End Sub

Private Sub lstWebsites_DblClick(ByVal Cancel As MSForms.ReturnBoolean)
     
    Call LaunchBrowser
  
End Sub

Private Sub lstWebsites_KeyDown(ByVal KeyCode As MSForms.ReturnInteger, ByVal Shift As Integer)

    If KeyCode = vbKeyReturn Then Call LaunchBrowser

End Sub

Private Sub cmdLaunchBrowser_Click()
    
    Call LaunchBrowser
  
End Sub

Private Sub cmdCancel_Click()

    Unload Me

End Sub

Private Sub cmdOpenWebsiteList_Click()
    'revise this to work without variable
    
    Dim dSourceDoc As Document
 
    Set dSourceDoc = Documents.Open(FileName:=ActiveDocument.AttachedTemplate.Path & Application.PathSeparator & sDATA_FILE, Visible:=True)
    dSourceDoc.Activate
    Unload Me
    
End Sub


Private Sub LaunchBrowser()
    
    Dim sSubject As String
    Dim sChosenSiteName As String
    Dim sChosenURL As String

    On Error GoTo Err_Msg

    sChosenSiteName = lstWebsites.Column(0)
    sChosenURL = lstWebsites.Column(1)
    If sChosenSiteName = "" Then Exit Sub
    sSubject = RemoveBadCharacters(TextSearchString.Text) 'sSelection
    sSubject = Replace(sSubject, " ", "%20")
    sSubject = Replace(sSubject, "#", "%23")
    sSubject = Replace(sSubject, "&", "%26")
    sSubject = Replace(sSubject, ChrW(8217), "%27") ' "'"
    ActiveDocument.FollowHyperlink Address:=sChosenURL & sSubject
    Unload Me

Exit Sub

Err_Msg:

    If Err.Number = 4198 Then
        MsgBox sChosenSiteName & " returns " & Chr(147) & sSelection & Chr$(148) & " as a bad search string." & Chr(13) & Chr(13) & "Please try with a different search string or using a different URL.", vbCritical, sDIALOGUE_TITLE
        Exit Sub
        Else
            MsgBox "The macro has encountered an error." & vbCrLf & Err.Number & ": " & Err.description, vbCritical, sDIALOGUE_TITLE
    End If
End Sub

